/*  
 *  XMMS is Copyright (C) 1998-2000  Peter Alm, Mikael Alm, Olle Hallnas, Thomas Nilsson and 4Front Technologies
 *  
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 */
#include "bte.h"
#include <stdlib.h>

#define BTE_MAX_BAUD_RATE 115200

static gboolean keepConfGoing;
gboolean bteconf_is_going = FALSE;
static GtkWidget *bteconf_mainwin = NULL, *bteconf_controlwin, *bteconf_playlistwin;
static GtkWidget *dev_entry, *btecode_entry, *playlist_entry, *playlist_spin;
static GtkWidget *baudrate_entry;

static gchar *bte_control[13] =
{
	N_("Play"),	N_("Stop"),	N_("Pause"),
	N_("Prev"),	N_("Next"),	N_("Vol +"),
	N_("Seek -5s"),	N_("Seek +5s"),	N_("Vol -"),
	N_("Shuffle"),	N_("Repeat"),	N_("Playlist"),
	N_("+100")
};
static gchar *bte_playlist[10] =
{
	N_("0"), N_("1"), N_("2"), N_("3"), N_("4"),
	N_("5"), N_("6"), N_("7"), N_("8"), N_("9")
};
static gchar *irbutton_to_edit;
static gint bte_was_enabled;


static void bteconf_ok_cb(GtkWidget * w, gpointer data)
{
	btecfg.device = g_strdup(gtk_entry_get_text(GTK_ENTRY(dev_entry)));
	btecfg.baudrate = atoi(gtk_entry_get_text(GTK_ENTRY(baudrate_entry)));
	if(btecfg.baudrate > BTE_MAX_BAUD_RATE)
	    btecfg.baudrate = BTE_MAX_BAUD_RATE;
	if(btecfg.baudrate < 0)
	    btecfg.baudrate = 9600;
	/* Re-initialize remote using new settings */
	bte_close_port();
	bte_open_port(btecfg.device);
	bte_save_config();
	gtk_widget_destroy(bteconf_mainwin);
}

static void bteconf_cancel_cb(GtkWidget * w, gpointer data)
{
	bte_read_config();
	gtk_widget_destroy(bteconf_mainwin);
}

static gint bteconf_codeentry_routine(gpointer data)
{
  char keycode[5];

  if (bte_poll_key(keycode)) {
    gtk_entry_set_text(GTK_ENTRY(btecode_entry), keycode);
  }

  if (keepConfGoing)
    return TRUE;
  else
    return FALSE;
}

static void bteconf_control_ok_cb(GtkWidget * w, gpointer data)
{
	gint i;

	keepConfGoing = FALSE;
	bte_close_port();
	if (bte_was_enabled)
		bte_open_port(btecfg.device);
	bteconf_is_going = FALSE;
	if (!strcmp(irbutton_to_edit, bte_control[0]))
		btecfg.button_play = g_strdup(gtk_entry_get_text(GTK_ENTRY(btecode_entry)));
	else if (!strcmp(irbutton_to_edit, bte_control[1]))
		btecfg.button_stop = g_strdup(gtk_entry_get_text(GTK_ENTRY(btecode_entry)));
	else if (!strcmp(irbutton_to_edit, bte_control[2]))
		btecfg.button_pause = g_strdup(gtk_entry_get_text(GTK_ENTRY(btecode_entry)));
	else if (!strcmp(irbutton_to_edit, bte_control[3]))
		btecfg.button_prev = g_strdup(gtk_entry_get_text(GTK_ENTRY(btecode_entry)));
	else if (!strcmp(irbutton_to_edit, bte_control[4]))
		btecfg.button_next = g_strdup(gtk_entry_get_text(GTK_ENTRY(btecode_entry)));
	else if (!strcmp(irbutton_to_edit, bte_control[5]))
		btecfg.button_volup = g_strdup(gtk_entry_get_text(GTK_ENTRY(btecode_entry)));
	else if (!strcmp(irbutton_to_edit, bte_control[6]))
		btecfg.button_seekb = g_strdup(gtk_entry_get_text(GTK_ENTRY(btecode_entry)));
	else if (!strcmp(irbutton_to_edit, bte_control[7]))
		btecfg.button_seekf = g_strdup(gtk_entry_get_text(GTK_ENTRY(btecode_entry)));
	else if (!strcmp(irbutton_to_edit, bte_control[8]))
		btecfg.button_voldown = g_strdup(gtk_entry_get_text(GTK_ENTRY(btecode_entry)));
	else if (!strcmp(irbutton_to_edit, bte_control[9]))
		btecfg.button_shuffle = g_strdup(gtk_entry_get_text(GTK_ENTRY(btecode_entry)));
	else if (!strcmp(irbutton_to_edit, bte_control[10]))
		btecfg.button_repeat = g_strdup(gtk_entry_get_text(GTK_ENTRY(btecode_entry)));
	else if (!strcmp(irbutton_to_edit, bte_control[11]))
		btecfg.button_playlist = g_strdup(gtk_entry_get_text(GTK_ENTRY(btecode_entry)));
	else if (!strcmp(irbutton_to_edit, bte_control[12]))
		btecfg.button_plus100 = g_strdup(gtk_entry_get_text(GTK_ENTRY(btecode_entry)));
	else
		for (i = 0; i < 10; i++)
			if (!strcmp(irbutton_to_edit, bte_playlist[i]))
				btecfg.button[i] = g_strdup(gtk_entry_get_text(GTK_ENTRY(btecode_entry)));

	gtk_widget_destroy(bteconf_controlwin);
}

static void bteconf_control_cancel_cb(GtkWidget * w, gpointer data)
{
	keepConfGoing = FALSE;
	bte_close_port();
	if (bte_was_enabled)
		bte_open_port(btecfg.device);
	bteconf_is_going = FALSE;
	gtk_widget_destroy(bteconf_controlwin);
}

static void bteconf_control_cb(GtkWidget * w, gchar * data)
{
	GtkWidget *vbox, *frame, *table, *button, *box;
	gint i;

	if (!bteconf_controlwin && !bteconf_playlistwin)
	{
		char *tmp;

		keepConfGoing = TRUE;
		irbutton_to_edit = data;
		bteconf_controlwin = gtk_window_new(GTK_WINDOW_DIALOG);
		gtk_signal_connect(GTK_OBJECT(bteconf_controlwin), "destroy", GTK_SIGNAL_FUNC(gtk_widget_destroyed), &bteconf_controlwin);
		tmp=g_strdup_printf(_("`%s' Button Setup"),_(data));
		gtk_window_set_title(GTK_WINDOW(bteconf_controlwin), tmp);
		g_free(tmp);
		gtk_window_set_policy(GTK_WINDOW(bteconf_controlwin), FALSE, FALSE, FALSE);
		gtk_window_set_position(GTK_WINDOW(bteconf_controlwin), GTK_WIN_POS_MOUSE);
		gtk_container_border_width(GTK_CONTAINER(bteconf_controlwin), 10);

		vbox = gtk_vbox_new(FALSE, 10);
		gtk_container_add(GTK_CONTAINER(bteconf_controlwin), vbox);

		frame = gtk_frame_new(_("Enter code or use remote"));
		gtk_container_border_width(GTK_CONTAINER(frame), 5);
		gtk_box_pack_start(GTK_BOX(vbox), frame, FALSE, FALSE, 0);

		table = gtk_table_new(1, 1, FALSE);
		gtk_container_set_border_width(GTK_CONTAINER(table), 5);
		gtk_container_add(GTK_CONTAINER(frame), table);
		gtk_table_set_row_spacings(GTK_TABLE(table), 2);
		gtk_table_set_col_spacings(GTK_TABLE(table), 5);

		btecode_entry = gtk_entry_new();
		gtk_table_attach_defaults(GTK_TABLE(table), btecode_entry, 0, 1, 0, 1);
		if (!strcmp(data, bte_control[0]))
			gtk_entry_set_text(GTK_ENTRY(btecode_entry), btecfg.button_play);
		else if (!strcmp(data, bte_control[1]))
			gtk_entry_set_text(GTK_ENTRY(btecode_entry), btecfg.button_stop);
		else if (!strcmp(data, bte_control[2]))
			gtk_entry_set_text(GTK_ENTRY(btecode_entry), btecfg.button_pause);
		else if (!strcmp(data, bte_control[3]))
			gtk_entry_set_text(GTK_ENTRY(btecode_entry), btecfg.button_prev);
		else if (!strcmp(data, bte_control[4]))
			gtk_entry_set_text(GTK_ENTRY(btecode_entry), btecfg.button_next);
		else if (!strcmp(data, bte_control[5]))
			gtk_entry_set_text(GTK_ENTRY(btecode_entry), btecfg.button_volup);
		else if (!strcmp(data, bte_control[6]))
			gtk_entry_set_text(GTK_ENTRY(btecode_entry), btecfg.button_seekb);
		else if (!strcmp(data, bte_control[7]))
			gtk_entry_set_text(GTK_ENTRY(btecode_entry), btecfg.button_seekf);
		else if (!strcmp(data, bte_control[8]))
			gtk_entry_set_text(GTK_ENTRY(btecode_entry), btecfg.button_voldown);
		else if (!strcmp(data, bte_control[9]))
			gtk_entry_set_text(GTK_ENTRY(btecode_entry), btecfg.button_shuffle);
		else if (!strcmp(data, bte_control[10]))
			gtk_entry_set_text(GTK_ENTRY(btecode_entry), btecfg.button_repeat);
		else if (!strcmp(data, bte_control[11]))
			gtk_entry_set_text(GTK_ENTRY(btecode_entry), btecfg.button_playlist);
		else if (!strcmp(data, bte_control[12]))
			gtk_entry_set_text(GTK_ENTRY(btecode_entry), btecfg.button_plus100);
		else
			for (i = 0; i < 10; i++)
				if (!strcmp(data, bte_playlist[i]))
					gtk_entry_set_text(GTK_ENTRY(btecode_entry), btecfg.button[i]);
		gtk_widget_show(btecode_entry);

		box = gtk_hbutton_box_new();
		gtk_button_box_set_layout(GTK_BUTTON_BOX(box), GTK_BUTTONBOX_END);
		gtk_button_box_set_spacing(GTK_BUTTON_BOX(box), 5);
		gtk_box_pack_start(GTK_BOX(vbox), box, FALSE, FALSE, 0);

		button = gtk_button_new_with_label(_("Ok"));
		gtk_signal_connect_object(GTK_OBJECT(button), "clicked", GTK_SIGNAL_FUNC(bteconf_control_ok_cb), NULL);
		GTK_WIDGET_SET_FLAGS(button, GTK_CAN_DEFAULT);
		gtk_box_pack_start(GTK_BOX(box), button, TRUE, TRUE, 0);
		gtk_widget_grab_default(button);
		gtk_widget_show(button);

		button = gtk_button_new_with_label(_("Cancel"));
		gtk_signal_connect_object(GTK_OBJECT(button), "clicked", GTK_SIGNAL_FUNC(bteconf_control_cancel_cb), NULL);
		GTK_WIDGET_SET_FLAGS(button, GTK_CAN_DEFAULT);
		gtk_box_pack_start(GTK_BOX(box), button, TRUE, TRUE, 0);
		gtk_widget_show(button);

		gtk_widget_show(box);
		gtk_widget_show(table);
		gtk_widget_show(frame);
		gtk_widget_show(vbox);
		gtk_widget_show(bteconf_controlwin);

		if ((bte_was_enabled = bte_get_portfd()))
			bte_close_port();
		bte_open_port(g_strdup(gtk_entry_get_text(GTK_ENTRY(dev_entry))));
		bteconf_is_going = TRUE;
		gtk_timeout_add(10, bteconf_codeentry_routine, NULL);
	}
}

static void spin_change_cb(GtkWidget * widget, GtkSpinButton * spin)
{
	gtk_entry_set_text(GTK_ENTRY(playlist_entry), btecfg.playlist[gtk_spin_button_get_value_as_int(spin)]);
}

static void pl_entry_change_cb(GtkWidget * widget, GtkSpinButton * spin)
{
	btecfg.playlist[gtk_spin_button_get_value_as_int(spin)] = g_strdup(gtk_entry_get_text(GTK_ENTRY(playlist_entry)));
}

void bte_configure() {
  GtkWidget *vbox, *notebook, *box, *frame, *table, *vbox2, *label,
    *button, *label_baudrate;
  GtkAdjustment *adj;
  gint i;
  
  bte_read_config();
  if (!bteconf_mainwin)
    {
      
      bteconf_mainwin = gtk_window_new(GTK_WINDOW_DIALOG);
      gtk_signal_connect(GTK_OBJECT(bteconf_mainwin), "destroy", GTK_SIGNAL_FUNC(gtk_widget_destroyed), &bteconf_mainwin);
      gtk_window_set_title(GTK_WINDOW(bteconf_mainwin), _("XMMS BTE Configuration"));
      gtk_window_set_policy(GTK_WINDOW(bteconf_mainwin), FALSE, FALSE, FALSE);
      gtk_window_set_position(GTK_WINDOW(bteconf_mainwin), GTK_WIN_POS_MOUSE);
      gtk_container_border_width(GTK_CONTAINER(bteconf_mainwin), 10);

      vbox = gtk_vbox_new(FALSE, 10);
      gtk_container_add(GTK_CONTAINER(bteconf_mainwin), vbox);
      
      notebook = gtk_notebook_new();
      gtk_box_pack_start(GTK_BOX(vbox), notebook, TRUE, TRUE, 0);
      
      box = gtk_vbox_new(FALSE, 5);
      gtk_container_set_border_width(GTK_CONTAINER(box), 5);
      
      frame = gtk_frame_new(_("Device:"));
      gtk_box_pack_start(GTK_BOX(box), frame, FALSE, FALSE, 0);

      table = gtk_table_new(2, 1, FALSE);
      gtk_container_set_border_width(GTK_CONTAINER(table), 5);
      gtk_container_add(GTK_CONTAINER(frame), table);
      gtk_table_set_row_spacings(GTK_TABLE(table), 5);
      gtk_table_set_col_spacings(GTK_TABLE(table), 5);
      
      label = gtk_label_new(_("Device: "));
      gtk_misc_set_alignment(GTK_MISC(label), 1.0, 0.5);
      gtk_table_attach_defaults(GTK_TABLE(table), label, 0, 1, 0, 1);
      gtk_widget_show(label);
      
      dev_entry = gtk_entry_new();
      gtk_entry_set_text(GTK_ENTRY(dev_entry), btecfg.device);
      gtk_table_attach_defaults(GTK_TABLE(table), dev_entry, 1, 2, 0, 1);
      gtk_widget_show(dev_entry);
      
      label_baudrate = gtk_label_new(_("Seral port speed: "));
      gtk_misc_set_alignment(GTK_MISC(label_baudrate), 1.0, 0.5);
      gtk_table_attach_defaults(GTK_TABLE(table), label_baudrate, 0, 1, 1, 2);
      gtk_widget_show(label_baudrate);
      
      baudrate_entry = gtk_entry_new();
      gtk_entry_set_text(GTK_ENTRY(baudrate_entry), g_strdup_printf("%d", btecfg.baudrate));
      gtk_table_attach_defaults(GTK_TABLE(table), baudrate_entry, 1, 2, 1, 2);
      gtk_widget_show(baudrate_entry);
      
      gtk_widget_show(table);
      gtk_widget_show(frame);
      
      frame = gtk_frame_new(_("Controls:"));
      gtk_box_pack_start(GTK_BOX(box), frame, FALSE, FALSE, 0);
      
      vbox2 = gtk_vbox_new(FALSE, 0);
      //              gtk_container_set_border_width(GTK_CONTAINER(vbox2),0);
      gtk_container_add(GTK_CONTAINER(frame), vbox2);
      
      table = gtk_table_new(5, 3, TRUE);
      gtk_container_set_border_width(GTK_CONTAINER(table), 5);
      gtk_table_set_row_spacings(GTK_TABLE(table), 5);
      gtk_table_set_col_spacings(GTK_TABLE(table), 5);
      gtk_box_pack_start(GTK_BOX(vbox2), table, FALSE, FALSE, 0);
      
      for (i = 0; i < 13; i++)
	{
	  button = gtk_button_new_with_label(_(bte_control[i]));
	  gtk_signal_connect(GTK_OBJECT(button), "clicked", GTK_SIGNAL_FUNC(bteconf_control_cb), bte_control[i]);
	  gtk_table_attach_defaults(GTK_TABLE(table), button, i % 3, (i % 3) + 1, i / 3, (i / 3) + 1);
	  gtk_widget_show(button);
	}
      gtk_widget_show(table);
      gtk_widget_show(frame);
      
      table = gtk_table_new(2, 5, FALSE);
      gtk_container_set_border_width(GTK_CONTAINER(table), 5);
      gtk_table_set_row_spacings(GTK_TABLE(table), 0);
      gtk_table_set_col_spacings(GTK_TABLE(table), 0);
      gtk_box_pack_start(GTK_BOX(vbox2), table, FALSE, FALSE, 0);
      
      for (i = 0; i < 10; i++)
		{
		  button = gtk_button_new_with_label(_(bte_playlist[i]));
		  gtk_signal_connect(GTK_OBJECT(button), "clicked", GTK_SIGNAL_FUNC(bteconf_control_cb), bte_playlist[i]);
		  gtk_table_attach_defaults(GTK_TABLE(table), button, i % 5, (i % 5) + 1, i / 5, (i / 5) + 1);
		  gtk_widget_show(button);
		}
      gtk_widget_show(table);
      gtk_widget_show(frame);

      frame = gtk_frame_new(_("Playlists:"));
      gtk_box_pack_start(GTK_BOX(box), frame, FALSE, FALSE, 0);
      
      table = gtk_table_new(2, 1, FALSE);
      gtk_container_set_border_width(GTK_CONTAINER(table), 5);
      gtk_container_add(GTK_CONTAINER(frame), table);
      gtk_table_set_row_spacings(GTK_TABLE(table), 5);
      gtk_table_set_col_spacings(GTK_TABLE(table), 5);
      
      adj = (GtkAdjustment *) gtk_adjustment_new(0, 0, 99, 1, 5, 0);
      playlist_spin = gtk_spin_button_new(adj, 0, 0);
      gtk_spin_button_set_numeric(GTK_SPIN_BUTTON(playlist_spin), TRUE);
      gtk_spin_button_set_update_policy(GTK_SPIN_BUTTON(playlist_spin), GTK_UPDATE_IF_VALID);
      gtk_spin_button_set_wrap(GTK_SPIN_BUTTON(playlist_spin), FALSE);
      gtk_table_attach_defaults(GTK_TABLE(table), playlist_spin, 0, 1, 0, 1);
      gtk_signal_connect(GTK_OBJECT(adj), "value_changed", GTK_SIGNAL_FUNC(spin_change_cb), (gpointer) playlist_spin);
      gtk_widget_show(playlist_spin);
      
      playlist_entry = gtk_entry_new();
      gtk_entry_set_text(GTK_ENTRY(playlist_entry), btecfg.playlist[0]);
      gtk_signal_connect(GTK_OBJECT(playlist_entry), "changed", GTK_SIGNAL_FUNC(pl_entry_change_cb), (gpointer) playlist_spin);
      gtk_table_attach_defaults(GTK_TABLE(table), playlist_entry, 1, 2, 0, 1);
      gtk_widget_show(playlist_entry);
      
      gtk_widget_show(table);
      gtk_widget_show(frame);
      gtk_widget_show(vbox2);
      gtk_widget_show(box);
      
      gtk_notebook_append_page(GTK_NOTEBOOK(notebook), box, gtk_label_new(_("General")));
     gtk_widget_show(notebook);

     box = gtk_hbutton_box_new();
     gtk_button_box_set_layout(GTK_BUTTON_BOX(box), GTK_BUTTONBOX_END);
     gtk_button_box_set_spacing(GTK_BUTTON_BOX(box), 5);
     gtk_box_pack_start(GTK_BOX(vbox), box, FALSE, FALSE, 0);
     
     button = gtk_button_new_with_label(_("Ok"));
     gtk_signal_connect(GTK_OBJECT(button), "clicked", GTK_SIGNAL_FUNC(bteconf_ok_cb), NULL);
     GTK_WIDGET_SET_FLAGS(button, GTK_CAN_DEFAULT);
     gtk_box_pack_start(GTK_BOX(box), button, TRUE, TRUE, 0);
     gtk_widget_grab_default(button);
     gtk_widget_show(button);
     
     button = gtk_button_new_with_label(_("Cancel"));
     gtk_signal_connect(GTK_OBJECT(button), "clicked", GTK_SIGNAL_FUNC(bteconf_cancel_cb), NULL);
     GTK_WIDGET_SET_FLAGS(button, GTK_CAN_DEFAULT);
     gtk_box_pack_start(GTK_BOX(box), button, TRUE, TRUE, 0);
     gtk_widget_show(button);
     gtk_widget_show(box);
     gtk_widget_show(vbox);
     gtk_widget_show(bteconf_mainwin); 
    }
}

