#include "bte.h"

#include <xmms/util.h>
#include <xmms/xmmsctrl.h>
#include <xmms/configfile.h>

#include "btio.h"
#include <stdio.h>
#include <errno.h>

static gboolean keepGoing = FALSE;
static pthread_t bte_thread;
static int key_pressed;
static char last_key[10];
static int reset_song = 1;
/* Config struct for device and buttons */
bteConfig btecfg;


void bte_read_config(void);
void bte_save_config(void);

static void *xmms_bte_routine(void *arg);
static void init(void);
static void cleanup(void);
static int create_menu();
static void set_title_on_phone(char* name, int time);

GeneralPlugin bte_gp =
{
        NULL,                   /* handle */
        NULL,                   /* filename */
        -1,                     /* xmms_session */
        NULL,                   /* Description */
        init,
        bte_about,
        bte_configure,
        cleanup
};


GeneralPlugin *get_gplugin_info(void)
{
	bte_gp.description = g_strdup_printf(_("BTE Control %s"), VERSION);
	return &bte_gp;
}

void bte_open_port(gchar * bt_port) {
  if (bt_open_port(bt_port) < 0) {
    perror("init port");
  } 
}

void bte_close_port(void) {
  if (bt_close_port() < 0) {
    perror("close port");
  }
}

int bte_get_portfd(void)
{
  return bt_getfd();
}

static void init(void)
{
	bte_read_config();	
	keepGoing = TRUE;
	pthread_create(&bte_thread, NULL, xmms_bte_routine, NULL);
}


static void cleanup(void) {
  keepGoing = FALSE;
  pthread_join(bte_thread, NULL);
}

void bte_read_config(void) {
  gint i;
  ConfigFile *cfgfile;
  gchar *filename, buttontext[20];

  btecfg.device = g_strdup("/dev/phone");
  btecfg.baudrate = 115200;
  for (i = 0; i < 10; i++)
    {
      btecfg.button[i] = g_strdup("");
      btecfg.playlist[i] = g_strdup("");
    }
  for (i = 10; i < 100; i++)
    btecfg.playlist[i] = g_strdup("");
  
  btecfg.button_play = g_strdup("^");
  btecfg.button_stop = g_strdup("v");
  btecfg.button_next = g_strdup(">");
  btecfg.button_prev = g_strdup("<");
  btecfg.button_pause = g_strdup("s");
  btecfg.button_seekf = g_strdup("8");
  btecfg.button_seekb = g_strdup("7");
  btecfg.button_volup = g_strdup("0");
  btecfg.button_voldown = g_strdup("*");
  btecfg.button_shuffle = g_strdup("5");
  btecfg.button_repeat = g_strdup("9");
  btecfg.button_playlist = g_strdup("");
  btecfg.button_plus100 = g_strdup("");

  filename = g_strconcat(g_get_home_dir(), "/.xmms/config", NULL);
  cfgfile = xmms_cfg_open_file(filename);
  if (cfgfile)
    {
      xmms_cfg_read_string(cfgfile, "bte", "device", &btecfg.device);
      xmms_cfg_read_int(cfgfile, "bte", "baudrate", &btecfg.baudrate);
      for (i = 0; i < 10; i++)
	{
	  sprintf(buttontext, "button%d", i);
	  xmms_cfg_read_string(cfgfile, "bte", buttontext, &btecfg.button[i]);
	  sprintf(buttontext, "playlist%d", i);
	  xmms_cfg_read_string(cfgfile, "bte", buttontext, &btecfg.playlist[i]);
	}
      for (i = 10; i < 100; i++)
	{
	  sprintf(buttontext, "playlist%d", i);
	  xmms_cfg_read_string(cfgfile, "bte", buttontext, &btecfg.playlist[i]);
	}
      xmms_cfg_read_string(cfgfile, "bte", "button_play", &btecfg.button_play);
      xmms_cfg_read_string(cfgfile, "bte", "button_stop", &btecfg.button_stop);
      xmms_cfg_read_string(cfgfile, "bte", "button_next", &btecfg.button_next);
      xmms_cfg_read_string(cfgfile, "bte", "button_prev", &btecfg.button_prev);
      xmms_cfg_read_string(cfgfile, "bte", "button_pause", &btecfg.button_pause);
      xmms_cfg_read_string(cfgfile, "bte", "button_seekf", &btecfg.button_seekf);
      xmms_cfg_read_string(cfgfile, "bte", "button_seekb", &btecfg.button_seekb);
      xmms_cfg_read_string(cfgfile, "bte", "button_volup", &btecfg.button_volup);
      xmms_cfg_read_string(cfgfile, "bte", "button_voldown", &btecfg.button_voldown);
      xmms_cfg_read_string(cfgfile, "bte", "button_shuffle", &btecfg.button_shuffle);
      xmms_cfg_read_string(cfgfile, "bte", "button_repeat", &btecfg.button_repeat);
      xmms_cfg_read_string(cfgfile, "bte", "button_playlist", &btecfg.button_playlist);
      xmms_cfg_read_string(cfgfile, "bte", "button_plus100", &btecfg.button_plus100);
      xmms_cfg_free(cfgfile);
    }
  g_free(filename);
}

void bte_save_config(void) {
  ConfigFile *cfgfile;
  gchar *filename, buttontext[20];
  gint i;

  filename = g_strconcat(g_get_home_dir(), "/.xmms/config", NULL);
  cfgfile = xmms_cfg_open_file(filename);
  if (!cfgfile)
    cfgfile = xmms_cfg_new();
  xmms_cfg_write_string(cfgfile, "bte", "device", btecfg.device);
  xmms_cfg_write_int(cfgfile, "bte", "baudrate", btecfg.baudrate);
  for (i = 0; i < 10; i++)
    {
      sprintf(buttontext, "button%d", i);
      xmms_cfg_write_string(cfgfile, "bte", buttontext, btecfg.button[i]);
      sprintf(buttontext, "playlist%d", i);
      xmms_cfg_write_string(cfgfile, "bte", buttontext, btecfg.playlist[i]);
    }
  for (i = 10; i < 100; i++)
    {
      sprintf(buttontext, "playlist%d", i);
      xmms_cfg_write_string(cfgfile, "bte", buttontext, btecfg.playlist[i]);
    }
  xmms_cfg_write_string(cfgfile, "bte", "button_play", btecfg.button_play);
  xmms_cfg_write_string(cfgfile, "bte", "button_stop", btecfg.button_stop);
  xmms_cfg_write_string(cfgfile, "bte", "button_next", btecfg.button_next);
  xmms_cfg_write_string(cfgfile, "bte", "button_prev", btecfg.button_prev);
  xmms_cfg_write_string(cfgfile, "bte", "button_pause", btecfg.button_pause);
  xmms_cfg_write_string(cfgfile, "bte", "button_seekf", btecfg.button_seekf);
  xmms_cfg_write_string(cfgfile, "bte", "button_seekb", btecfg.button_seekb);
  xmms_cfg_write_string(cfgfile, "bte", "button_volup", btecfg.button_volup);
  xmms_cfg_write_string(cfgfile, "bte", "button_voldown", btecfg.button_voldown);
  xmms_cfg_write_string(cfgfile, "bte", "button_shuffle", btecfg.button_shuffle);
  xmms_cfg_write_string(cfgfile, "bte", "button_repeat", btecfg.button_repeat);
  xmms_cfg_write_string(cfgfile, "bte", "button_playlist", btecfg.button_playlist);
  xmms_cfg_write_string(cfgfile, "bte", "button_plus100", btecfg.button_plus100);

  xmms_cfg_write_file(cfgfile, filename);
  xmms_cfg_free(cfgfile);
  g_free(filename);
}

#define SEND_CMD(command, answer) \
    bt_put_command (command, answer, sizeof (answer), 5, NULL);
  
#define IS_OK(answer) (strstr(answer, "OK") != NULL)


int bte_poll_key(char* keycode) {
  char answer[1024], *key;

  if (bt_read_tmo(answer, sizeof(answer), 10) < 0) 
    return 0;

  if ((key = strstr(answer, "+CKEV:"))) {
      keycode[0] = key[6];
      keycode[1] = 0;
      return 1;
    }
  return 0;
}

static void handle_key_press(char* key, int real) {
  char* text = key;
  //printf("key press=%s\n", key);

  if (real) {
    key_pressed = 1;
    strcpy(last_key, key);
  }

  // just skip everything beside char
  text[1] = 0;

  if (!strcmp(text, btecfg.button_play))
    {
      xmms_remote_play(bte_gp.xmms_session);
    }
  
  else if (!strcmp(text, btecfg.button_stop))
    {
      xmms_remote_stop(bte_gp.xmms_session);
    }
  else if (!strcmp(text, btecfg.button_pause))
    {
      xmms_remote_pause(bte_gp.xmms_session);
    }
  else if (!strcmp(text, btecfg.button_shuffle))
    {
      xmms_remote_toggle_shuffle(bte_gp.xmms_session);
    }
  else if (!strcmp(text, btecfg.button_repeat))
    {
      xmms_remote_toggle_repeat(bte_gp.xmms_session);
    }
  else if (!strcmp(text, btecfg.button_next))
    {
      xmms_remote_playlist_next(bte_gp.xmms_session);
    }
  else if (!strcmp(text, btecfg.button_prev))
    {
      xmms_remote_playlist_prev(bte_gp.xmms_session);
    }
  else if (!strcmp(text, btecfg.button_seekf))
    {
      gint playlist_time, output_time,  playlist_pos;

      playlist_pos = xmms_remote_get_playlist_pos(bte_gp.xmms_session);
      playlist_time = xmms_remote_get_playlist_time(bte_gp.xmms_session, 
						    playlist_pos);
      output_time = xmms_remote_get_output_time(bte_gp.xmms_session);
      if (playlist_time - output_time < 5000)
	output_time = playlist_time - 5000;
      xmms_remote_jump_to_time(bte_gp.xmms_session, output_time + 5000);
      
    }
  else if (!strcmp(text, btecfg.button_seekb))
    {
      gint  output_time;

      output_time = xmms_remote_get_output_time(bte_gp.xmms_session);
      if (output_time < 5000)
	output_time = 5000;
      xmms_remote_jump_to_time(bte_gp.xmms_session, output_time - 5000);
    }
  else if (!strcmp(text, btecfg.button_volup))
    {
      gint vr, vl;

      xmms_remote_get_volume(bte_gp.xmms_session, &vl, &vr);
      if (vl > 95)
	vl = 95;
      if (vr > 95)
	vr = 95;
      xmms_remote_set_volume(bte_gp.xmms_session, vl + 5, vr + 5);
    }
  else if (!strcmp(text, btecfg.button_voldown))
    {
      gint vr, vl;
      
      xmms_remote_get_volume(bte_gp.xmms_session, &vl, &vr);
      if (vl < 5)
	vl = 5;
      if (vr < 5)
	vr = 5;
      xmms_remote_set_volume(bte_gp.xmms_session, vl - 5, vr - 5);
    }
}


static void handle_key_release(char* key, int real) {
  //printf("key release=%s\n", key);
  if (real)
    key_pressed = 0;
}

static void handle_command(char* cmd) {
  char answer[1024];
  char *prev, *next, *key;
 
  //skip lines starting with \n and \r
  if (cmd[0] == '\r') cmd++;
  if (cmd[0] == '\n') cmd++;
   // most common case
  if (!cmd[0]) return;
  
  // if recieved multiline command - handle line by line and return
  prev = cmd;
  next = strchr(cmd, '\r');
  if (next) {
    char copy[1024];
    int len;
    do {     
      len = next-prev;
      if (len > 2) {
	memcpy(copy, prev, len);
	copy[len] = 0;
	handle_command(copy);
      }
    prev = next+1;
    } while ((next = strchr(prev, '\r'))) ;
    // now return
    return;
  }

  // create widget to recieve phone key presses
  if (memcmp(cmd, "*EAAI", 5) == 0) {
    reset_song = 1;
    set_title_on_phone("XMMS Remote, use joystick and keyboard to control playback", -1);
    return;
  }

  // "No" pressed/released - dismiss  
  if (memcmp(cmd, "+CKEV: e", 8)  == 0) {
    // finish events forwarding
    SEND_CMD("AT+CMER=0,0,0,0,0\r", answer);
    handle_command(answer);
    
    // and resend No back
    SEND_CMD("AT+CKPD=\"e\"\r", answer);
    
    return;
  }

  // here we handle different keypresses
  if (memcmp(cmd, "+CKEV:", 6) == 0) {
    key = cmd+7;
    if (key[2] == '1') 
      handle_key_press(key, 1);
    else if (key[2] == '0') 
      handle_key_release(key, 1);
    else
      printf("something wrong: key should be pressed or released: cmd=%s\n",
	     cmd);
    return;
  }

  // printf("unhandled cmd %s %d\n", cmd, cmd[0]);
}

static int read_command(char* buf, int l) {
  int ch, len = 0;
  char* p = buf;

  while (len < l) {
    ch = bt_readchar(100);
    if (ch >= 0 && ch != '\r'){
      *p++ = (char)ch;
      len++;
    } else {
      break;
    }
  }
  
  buf[len] = 0;
  return len;
}


static void set_title_on_phone(char* name, int time) {
  char buf[1024], answer[1024], *title;
  int buf_len = sizeof(buf)-10;
  
  SEND_CMD("AT+CMER=3,2,0,0,0\r", answer);
  handle_command(answer);

  sprintf(buf, "AT*EAID=14,8,\"Remote\",\"");  

  // escape " in title - we don't care about name modification, as 
  // it's our copy

  title = name;
  while (*title) {
    if (*title == '"') *title = '\'';
    title++;
  }
  
  // now put name to the buffer  
  title = &buf[0] + strlen(buf);

  if (time != -1) {
    snprintf(title, buf_len, "%s [%d:%02d]  ", name,
	     time / 60000, (time / 1000) % 60);
  } else {
    snprintf(title, buf_len, "%s", name);
  }
  sprintf(buf+strlen(buf), "\"\r");
  //printf("current is %s\n", buf);

  SEND_CMD(buf, answer);
  handle_command(answer);
}

static void *xmms_bte_routine(void *arg) {
  char answer[1024];
  GTimer *timer1, *timer2;

  timer1 = g_timer_new();
  timer2 = g_timer_new();
  
  bte_open_port(btecfg.device);

  if (create_menu() < 0) goto done;

  key_pressed = 0;

  int current_song = -1;

  while (keepGoing)
    {      
      // so config dialog could read keypresses
      if (!bteconf_is_going)
	{
	  if (read_command(answer, sizeof(answer))) {
	    handle_command(answer);	    
	  }

	  if (key_pressed == 0) {
	      g_timer_reset(timer1);
	    } else {
	      if (key_pressed && g_timer_elapsed(timer1, NULL) > 0.2) {
		handle_key_press(last_key, 0);
		g_timer_reset(timer1);
	      }
	    }

	  if (g_timer_elapsed(timer2, NULL) > 0.3) {
	    g_timer_reset(timer2);

	    if (reset_song) {
	      reset_song = 0;
	      current_song = -1;
	    }

	    if (xmms_remote_is_playing(bte_gp.xmms_session)) {	      
	      
	      int song = xmms_remote_get_playlist_pos(bte_gp.xmms_session);
	      if (song != current_song) {	      
		char* current_name 
		  = xmms_remote_get_playlist_title(bte_gp.xmms_session, song);
		current_song = song;
		set_title_on_phone(current_name, 
				   xmms_remote_get_playlist_time(bte_gp.xmms_session, song));
		g_free(current_name);
	      }	    
	    } else {
	      if (current_song != -1) {
		current_song = -1;
		set_title_on_phone("XMMS Remote, use joystick and keyboard to control playback", -1);
	      }
	    }
	  }
	} else {
	  // reset pressed key
	  key_pressed = 0;
	}
      usleep(50000);
    }

 done:  
  g_timer_destroy(timer1);	 
  bte_close_port();
  pthread_exit(NULL);
  return NULL;
}



static int create_menu(void) {
  int rv = 0;
  char answer[1024];

  // bad device
  if (bte_get_portfd() <= 0) return -1; 
  // we don't need echoing of our input
  SEND_CMD("ATE0\r", answer)
  SEND_CMD("AT+CSCS=\"8859-1\"\r", answer);
  SEND_CMD("AT*EAM=\"XMMS remote\"\r", answer);
  handle_command(answer);
  return rv;
} 
